package com.maenle.bump.util

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.google.firebase.ktx.Firebase
import com.google.firebase.messaging.FirebaseMessaging
import org.json.JSONArray
import org.json.JSONObject
import java.io.File

class BumpProcessor constructor(context: Context) {

    private var rest: RestSingleton = RestSingleton.getInstance(context)
    private var secret: String? = null
    private lateinit var log: JSONArray
    private var messenger: MessageProcessor? = null

    init {
        secret = getSecret(context)
        log = getLog(context)
        messenger = secret?.let { MessageProcessor(it) }
    }

    companion object {
        @Volatile
        private var INSTANCE: BumpProcessor? = null
        fun getInstance(context: Context) =
            INSTANCE ?: synchronized(this) {
                INSTANCE ?: BumpProcessor(context).also {
                    INSTANCE = it
                }
            }
    }

    private fun updateLog(context: Context, list: JSONArray) {
        for(i in 0 until list.length()) {
            var exists = false
            for(j in 0 until log.length()) {
                if (list[i] == log[j]) {
                    exists = true
                    continue
                }
            }
            exists.let {
                log.put(list[i])
                addToLog(context, list[i] as JSONObject)
            }
        }
    }

    fun hasSecret(): Boolean {
        secret?.let {
            return true
        }
        return false
    }

    private fun getSecret(context: Context):String? {
        val local = LocalData(context)
        return local.code
    }

    fun startUpdateHandler(context: Context) {

        val delay: Long = 30_000 //milliseconds
        val handler = Handler(Looper.getMainLooper())

        handler.postDelayed(object : Runnable {
            override fun run() {
                updateFromServer(context)
                handler.postDelayed(this, delay)
            }
        }, delay)
    }

    private fun updateFromServer(context: Context) {
        messenger?.let {rest.list(it.sender) {list -> updateLog(context, list)}}
    }

    private fun addToLog(context: Context, line: JSONObject) {
        val logFile = File(context.filesDir, ".bump_log")
        logFile.appendText(line.toString())
    }

    fun addSecret(context: Context, newCode: String) {
        val local = LocalData(context)
        local.code = newCode
        secret = getSecret(context)

        setFirebaseToken()
    }

    private fun getLog(context: Context): JSONArray {
        val logFile = File(context.filesDir, ".bump_log")
        val log = JSONArray()
        !logFile.exists().let {
            logFile.createNewFile()
        }
        for(line in logFile.readLines()) {
            log.put(JSONObject(line))
        }
        return log
    }

    private fun setFirebaseToken() {
        val tokenTask = FirebaseMessaging.getInstance().token
        tokenTask.addOnSuccessListener {  token -> (
                messenger?.let { rest.firebase(it.sender, token) {result -> Log.d("result", result.toString())}})
        }
    }

}